"""cari anime dan film subtitle Indonesia"""

__version__ = "1.5.35"
__author__ = "Val"
__license__ = "MIT"
__all__ = [
    "extractors",
    "Bypass"
]

from .cli import main
from .extractors.bypasser import Bypass



from . import BaseExtractor


class Melongmovie(BaseExtractor):
    tag = "movie"
    host = "https://melongmovie.net"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        meta["image"] = soup.find(class_="wp-post-image")["src"]
        meta["judul"] = self.re.split(
            "(?i)(?:bd )?(?:batch )?subtitle", soup.title.text)[0]
        alias = {
            "Country": "negara",
            "Quality": "kualitas",
            "Network": "jaringan",
            "Duration": "durasi",
            "Stars": "bintang film",
            "Release": "rilis"
        }
        if (ul := soup.find("ul", class_="data")):
            for li in ul.findAll("li"):
                k, v = self.re.split(r"\s*:\s*", li.text)
                meta.add(alias.get(k, k), v)

        sinopsis = soup.find(class_="dzdesu").findPrevious("p")
        meta["sinopsis"] = sinopsis.text

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        for ep in soup.findAll(text=self.re.compile(r"(?i)episode\s+\d+|LINK DOWNLOAD")):
            content = ep.findNext("div")
            r = {}
            for p in content.findAll("p"):
                if p.a:
                    y = {}
                    for a in p.findAll("a"):
                        y[a.text] = a["href"]
                    title = self.re.search(r"\s*([^=]+)", p.text)
                    r[title.group(1)] = y
            if r:
                result[ep] = r

        for ep in soup.findAll("h2", text=self.re.compile(r"(?i)episode\s+\d+")):
            r = {}

            ul = ep.findNext("ul")
            for li in ul.findAll("li"):
                sub = "/".join(strong.text for strong in li.findAll("strong"))
                if sub.count("/") > 2:
                    continue

                y = {}
                for a in li.findAll("a"):
                    y[a.text] = a["href"]
                r[sub] = y
            result[ep.text] = r

        pattern = self.re.compile(r"[A-Z ]+:")
        if (ref := soup.find("strong", text=pattern)):
            for li in ref.findAllNext("li"):
                sub = "/".join(strong.text for strong in li.findAll("strong"))
                r = {}
                for a in li.findAll("a"):
                    r[a.text] = a["href"]

                title = li.findPrevious(
                    "strong", text=pattern).text.strip(": \n")
                if not result.get(title):
                    result[title] = {}
                result[title][sub] = r
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(f"{self.host}/page/{page}",
                               params={"s": query})
        soup = self.soup(raw)

        result = []
        if (los := soup.find(class_="los")):
            for article in los.findAll("article"):
                a = article.find("a")
                r = {
                    "id": self.getPath(a["href"]),
                    "title": a["alt"]
                }

                for k in ("quality", "eps"):
                    if (i := article.find(class_=k)):
                        r[k] = i.text
                for ip in ("genre", "name"):
                    if (i := article.findAll(itemprop=ip)):
                        r[ip] = [a.text for a in i]
                result.append(r)
        return result



from . import BaseExtractor


class Nekonime(BaseExtractor):
    host = "https://nekonime.stream"
    tag = "anime"
    required = ["proxy"]

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        content = soup.find(id="info", role="tabpanel")

        alias = {"Skor": "score", "Diterbitkan Nekonime": "diunggah"}
        for div in content.findAll(style="margin-bottom: 7px;"):
            k, v = self.re.split(r"\s*:\s*", div.text)
            meta.add(alias.get(k, k), v)

        sinop = soup.find(id="sinopsis")
        sinop.h2.decompose()
        meta["sinopsis"] = sinop.text

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        d = {}
        if (eps := soup.find(class_="daftarepi")):
            for a in eps.findAll("a"):
                d[a.text] = "re:" + self.getPath(a["href"])
            return d

        if (eps := soup.find(class_="flircontainer")):
            alleps = eps.find("a", text="All Eps")
            d[alleps.text] = "re:" + self.getPath(alleps["href"])

        if (ddl := soup.find(class_="soraddl")):
            for p in ddl.findAll(class_="soraurl"):
                result = {}
                for a in p.findAll("a"):
                    result[a.text] = a["href"]
                d[p.strong.text] = result
        return d

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={"s": query})
        soup = self.soup(raw)

        result = []
        for article in soup.findAll(class_="article-body"):
            a = article.find("a")
            result.append({
                "title": a.text,
                "id": self.getPath(a["href"])
            })
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of exrex.
#
# exrex is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# exrex is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with exrex. If not, see < http://www.gnu.org/licenses/ >.
#
# (C) 2012- by Adam Tauber, <asciimoo@gmail.com>

try:
    from future_builtins import map, range
except:
    pass
from re import sre_parse, U
from itertools import tee
from random import choice, randint
from types import GeneratorType

from sys import version_info
IS_PY3 = version_info[0] == 3
IS_PY36_OR_GREATER = IS_PY3 and version_info[1] > 5

if IS_PY3:
    unichr = chr

__all__ = (
    'generate',
    'CATEGORIES',
    'count',
    'parse',
    'getone',
    'sre_to_string',
    'simplify'
)

CATEGORIES = {
    sre_parse.CATEGORY_SPACE: sorted(sre_parse.WHITESPACE),
    sre_parse.CATEGORY_DIGIT: sorted(sre_parse.DIGITS),
    'category_any': [unichr(x) for x in range(32, 123)]
}


def _build_reverse_categories():
    reverse = {}
    for key, value in sre_parse.CATEGORIES.items():
        if not hasattr(value[1], '__iter__'):
            continue

        for vv in value[1]:
            if value[0] == sre_parse.IN and vv[0] == sre_parse.CATEGORY:
                reverse.update({vv[1]: key})

    return reverse


REVERSE_CATEGORIES = _build_reverse_categories()


def comb(g, i):
    for c in g:
        g2, i = tee(i)
        for c2 in g2:
            yield c + c2


def mappend(g, c):
    for cc in g:
        yield cc + c


def dappend(g, d, k):
    for cc in g:
        yield cc + d[k]


def _in(d):
    ret = []
    neg = False
    for i in d:
        if i[0] == sre_parse.RANGE:
            subs = map(unichr, range(i[1][0], i[1][1] + 1))
            if neg:
                for char in subs:
                    try:
                        ret.remove(char)
                    except:
                        pass
            else:
                ret.extend(subs)
        elif i[0] == sre_parse.LITERAL:
            if neg:
                try:
                    ret.remove(unichr(i[1]))
                except:
                    pass
            else:
                ret.append(unichr(i[1]))
        elif i[0] == sre_parse.CATEGORY:
            subs = CATEGORIES.get(i[1], [''])
            if neg:
                for char in subs:
                    try:
                        ret.remove(char)
                    except:
                        pass
            else:
                ret.extend(subs)
        elif i[0] == sre_parse.NEGATE:
            ret = list(CATEGORIES['category_any'])
            neg = True
    return ret


def prods(orig, ran, items, limit, grouprefs):
    for o in orig:
        for r in ran:
            if r == 0:
                yield o
            else:
                ret = [o]
                for _ in range(r):
                    ret = ggen(
                        ret, _gen, items, limit=limit, count=False, grouprefs=grouprefs)
                for i in ret:
                    yield i


def ggen(g1, f, *args, **kwargs):
    groupref = None
    grouprefs = kwargs.get('grouprefs', {})
    if 'groupref' in kwargs.keys():
        groupref = kwargs.pop('groupref')
    for a in g1:
        g2 = f(*args, **kwargs)
        if isinstance(g2, GeneratorType):
            for b in g2:
                grouprefs[groupref] = b
                yield a + b
        else:
            yield g2


def concit(g1, seqs, limit, grouprefs):
    for a in g1:
        for s in seqs:
            for b in _gen(s, limit, grouprefs=grouprefs):
                yield a + b


def _gen(d, limit=20, count=False, grouprefs=None):
    """docstring for _gen"""
    if grouprefs is None:
        grouprefs = {}
    ret = ['']
    strings = 0
    literal = False
    for i in d:
        if i[0] == sre_parse.IN:
            subs = _in(i[1])
            if count:
                strings = (strings or 1) * len(subs)
            ret = comb(ret, subs)
        elif i[0] == sre_parse.LITERAL:
            literal = True
            ret = mappend(ret, unichr(i[1]))
        elif i[0] == sre_parse.CATEGORY:
            subs = CATEGORIES.get(i[1], [''])
            if count:
                strings = (strings or 1) * len(subs)
            ret = comb(ret, subs)
        elif i[0] == sre_parse.ANY:
            subs = CATEGORIES['category_any']
            if count:
                strings = (strings or 1) * len(subs)
            ret = comb(ret, subs)
        elif i[0] == sre_parse.MAX_REPEAT or i[0] == sre_parse.MIN_REPEAT:
            items = list(i[1][2])
            if i[1][1] + 1 - i[1][0] >= limit:
                r1 = i[1][0]
                r2 = i[1][0] + limit
            else:
                r1 = i[1][0]
                r2 = i[1][1] + 1
            ran = range(r1, r2)
            if count:
                branch_count = 0
                for p in ran:
                    branch_count += pow(_gen(items, limit, True, grouprefs), p)
                strings = (strings or 1) * branch_count

            ret = prods(ret, ran, items, limit, grouprefs)
        elif i[0] == sre_parse.BRANCH:
            if count:
                for x in i[1][1]:
                    strings += _gen(x, limit, True, grouprefs) or 1
            ret = concit(ret, i[1][1], limit, grouprefs)
        elif i[0] == sre_parse.SUBPATTERN or i[0] == sre_parse.ASSERT:
            subexpr = i[1][1]
            if IS_PY36_OR_GREATER and i[0] == sre_parse.SUBPATTERN:
                subexpr = i[1][3]
            if count:
                strings = (
                    strings or 1) * (sum(ggen([0], _gen, subexpr, limit=limit, count=True, grouprefs=grouprefs)) or 1)
            ret = ggen(ret, _gen, subexpr, limit=limit, count=False,
                       grouprefs=grouprefs, groupref=i[1][0])
        # ignore ^ and $
        elif i[0] == sre_parse.AT:
            continue
        elif i[0] == sre_parse.NOT_LITERAL:
            subs = list(CATEGORIES['category_any'])
            if unichr(i[1]) in subs:
                subs.remove(unichr(i[1]))
            if count:
                strings = (strings or 1) * len(subs)
            ret = comb(ret, subs)
        elif i[0] == sre_parse.GROUPREF:
            ret = dappend(ret, grouprefs, i[1])
        elif i[0] == sre_parse.ASSERT_NOT:
            pass
        else:
            print('[!] cannot handle expression ' + repr(i))

    if count:
        if strings == 0 and literal:
            inc = True
            for i in d:
                if i[0] not in (sre_parse.AT, sre_parse.LITERAL):
                    inc = False
            if inc:
                strings = 1
        return strings

    return ret


def _randone(d, limit=20, grouprefs=None):
    if grouprefs is None:
        grouprefs = {}
    """docstring for _randone"""
    ret = ''
    for i in d:
        if i[0] == sre_parse.IN:
            ret += choice(_in(i[1]))
        elif i[0] == sre_parse.LITERAL:
            ret += unichr(i[1])
        elif i[0] == sre_parse.CATEGORY:
            ret += choice(CATEGORIES.get(i[1], ['']))
        elif i[0] == sre_parse.ANY:
            ret += choice(CATEGORIES['category_any'])
        elif i[0] == sre_parse.MAX_REPEAT or i[0] == sre_parse.MIN_REPEAT:
            if i[1][1] + 1 - i[1][0] >= limit:
                min, max = i[1][0], i[1][0] + limit - 1
            else:
                min, max = i[1][0], i[1][1]
            for _ in range(randint(min, max)):
                ret += _randone(list(i[1][2]), limit, grouprefs)
        elif i[0] == sre_parse.BRANCH:
            ret += _randone(choice(i[1][1]), limit, grouprefs)
        elif i[0] == sre_parse.SUBPATTERN or i[0] == sre_parse.ASSERT:
            subexpr = i[1][1]
            if IS_PY36_OR_GREATER and i[0] == sre_parse.SUBPATTERN:
                subexpr = i[1][3]
            subp = _randone(subexpr, limit, grouprefs)
            if i[1][0]:
                grouprefs[i[1][0]] = subp
            ret += subp
        elif i[0] == sre_parse.AT:
            continue
        elif i[0] == sre_parse.NOT_LITERAL:
            c = list(CATEGORIES['category_any'])
            if unichr(i[1]) in c:
                c.remove(unichr(i[1]))
            ret += choice(c)
        elif i[0] == sre_parse.GROUPREF:
            ret += grouprefs[i[1]]
        elif i[0] == sre_parse.ASSERT_NOT:
            pass
        else:
            print('[!] cannot handle expression "%s"' % str(i))

    return ret


def sre_to_string(sre_obj, paren=True):
    """sre_parse object to string

    :param sre_obj: Output of sre_parse.parse()
    :type sre_obj: list
    :rtype: str
    """
    ret = u''
    for i in sre_obj:
        if i[0] == sre_parse.IN:
            prefix = ''
            if len(i[1]) and i[1][0][0] == sre_parse.NEGATE:
                prefix = '^'
            ret += u'[{0}{1}]'.format(prefix, sre_to_string(i[1], paren=paren))
        elif i[0] == sre_parse.LITERAL:
            u = unichr(i[1])
            ret += u if u not in sre_parse.SPECIAL_CHARS else '\\{0}'.format(u)
        elif i[0] == sre_parse.CATEGORY:
            ret += REVERSE_CATEGORIES[i[1]]
        elif i[0] == sre_parse.ANY:
            ret += '.'
        elif i[0] == sre_parse.BRANCH:
            # TODO simplifications here
            parts = [sre_to_string(x, paren=paren) for x in i[1][1]]
            if not any(parts):
                continue
            if i[1][0]:
                if len(parts) == 1:
                    paren = False
                prefix = ''
            else:
                prefix = '?:'
            branch = '|'.join(parts)
            if paren:
                ret += '({0}{1})'.format(prefix, branch)
            else:
                ret += '{0}'.format(branch)
        elif i[0] == sre_parse.SUBPATTERN:
            subexpr = i[1][1]
            if IS_PY36_OR_GREATER and i[0] == sre_parse.SUBPATTERN:
                subexpr = i[1][3]
            if i[1][0]:
                ret += '({0})'.format(sre_to_string(subexpr, paren=False))
            else:
                ret += '{0}'.format(sre_to_string(subexpr, paren=paren))
        elif i[0] == sre_parse.NOT_LITERAL:
            ret += '[^{0}]'.format(unichr(i[1]))
        elif i[0] == sre_parse.MAX_REPEAT:
            if i[1][0] == i[1][1]:
                range_str = '{{{0}}}'.format(i[1][0])
            else:
                if i[1][0] == 0 and i[1][1] - i[1][0] == sre_parse.MAXREPEAT:
                    range_str = '*'
                elif i[1][0] == 1 and i[1][1] - i[1][0] == sre_parse.MAXREPEAT - 1:
                    range_str = '+'
                else:
                    range_str = '{{{0},{1}}}'.format(i[1][0], i[1][1])
            ret += sre_to_string(i[1][2], paren=paren) + range_str
        elif i[0] == sre_parse.MIN_REPEAT:
            if i[1][0] == 0 and i[1][1] == sre_parse.MAXREPEAT:
                range_str = '*?'
            elif i[1][0] == 1 and i[1][1] == sre_parse.MAXREPEAT:
                range_str = '+?'
            elif i[1][1] == sre_parse.MAXREPEAT:
                range_str = '{{{0},}}?'.format(i[1][0])
            else:
                range_str = '{{{0},{1}}}?'.format(i[1][0], i[1][1])
            ret += sre_to_string(i[1][2], paren=paren) + range_str
        elif i[0] == sre_parse.GROUPREF:
            ret += '\\{0}'.format(i[1])
        elif i[0] == sre_parse.AT:
            if i[1] == sre_parse.AT_BEGINNING:
                ret += '^'
            elif i[1] == sre_parse.AT_END:
                ret += '$'
        elif i[0] == sre_parse.NEGATE:
            pass
        elif i[0] == sre_parse.RANGE:
            ret += '{0}-{1}'.format(unichr(i[1][0]), unichr(i[1][1]))
        elif i[0] == sre_parse.ASSERT:
            if i[1][0]:
                ret += '(?={0})'.format(sre_to_string(i[1][1], paren=False))
            else:
                ret += '{0}'.format(sre_to_string(i[1][1], paren=paren))
        elif i[0] == sre_parse.ASSERT_NOT:
            pass
        else:
            print('[!] cannot handle expression "%s"' % str(i))
    return ret


def simplify(regex_string):
    """Simplify a regular expression

    :param regex_string: Regular expression
    :type regex_string: str
    :rtype: str
    """
    r = parse(regex_string)
    return sre_to_string(r)


def parse(s):
    """Regular expression parser

    :param s: Regular expression
    :type s: str
    :rtype: list
    """
    if IS_PY3:
        r = sre_parse.parse(s, flags=U)
    else:
        r = sre_parse.parse(s.decode('utf-8'), flags=U)
    return list(r)


def generate(s, limit=20):
    """Creates a generator that generates all matching strings to a given regular expression

    :param s: Regular expression
    :type s: str
    :param limit: Range limit
    :type limit: int
    :returns: string generator object
    """
    return _gen(parse(s), limit)


def count(s, limit=20):
    """Counts all matching strings to a given regular expression

    :param s: Regular expression
    :type s: str
    :param limit: Range limit
    :type limit: int
    :rtype: int
    :returns: number of matching strings
    """
    return _gen(parse(s), limit, count=True)


def getone(regex_string, limit=20):
    """Returns a random matching string to a given regular expression
    """
    return _randone(parse(regex_string), limit)



from . import BaseExtractor


class Kusonime(BaseExtractor):
    tag = "anime"
    host = "https://kusonime.com"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        alias = {
            'japanese': "judul alternatif",
            'seasons': 'musim',
            'producers': 'produser',
            'type': 'tipe',
            'duration': 'durasi',
            'released on': 'tayang'
        }
        meta = self.MetaSet()
        meta["image"] = soup.find(class_="wp-post-image")["src"]
        meta["judul"] = self.re.split(
            "(?i)(?:bd )?(?:batch )?subtitle", soup.title.text)[0]

        content = soup.find(class_="info")
        for p in content.findAll("p"):
            if ":" not in p.text:
                continue
            k, v = map(lambda x: x.lower().strip(),
                       self.re.split("\s*:\s*", p.text))
            k = alias.get(k, k)
            meta.add(k, v, split=k not in ["score", "tayang"])
        content.decompose()
        if (clear := soup.find(class_="clear")):
            sinop = []
            for p in clear.findAllNext("p"):
                if p.find("a"):
                    break
                sinop.append(p.text)
            meta["sinopsis"] = " ".join(sinop)

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        d = {}
        for n, smokeddl in enumerate(soup.findAll("div", class_="smokeddl")[:-1], start=1):
            title = f"{n}. " + smokeddl.find(class_="smokettl").text
            d[title] = {}
            for smokeurl in smokeddl.findAll(class_="smokeurl"):
                res = smokeurl.strong.text
                links = {}
                for a in smokeurl.findAll("a"):
                    links[a.text] = a["href"]
                d[title][res] = links
        return d

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(self.host + f"/page/{page}/", params={
            "s": query})
        soup = self.soup(raw)

        result = []
        for item in soup.findAll("div", class_="content"):
            d = {
                "title": item.h2.text,
                "id": self.getPath(item.a["href"])
            }
            for p in item.findAll("p"):
                if "genre" in p.text:
                    d["genre"] = [a.text for a in p.findAll(p)]
                elif "release" in p.text:
                    d["released"] = p.text.split("on ")[-1]
            result.append(d)
        return result



from . import BaseExtractor


class Dramaindo(BaseExtractor):
    tag = "movie"
    host = "https://k.dramaindo.my.id"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        content = soup.find(class_="post-wrapper single")
        meta.register(None, content.text)

        meta.setItem("title", "judul")
        meta.setItem("original title", "judul alternatif")
        meta.setItem("genre")
        meta.setItem("cast", "karakter")
        meta.setItem("year", "tahun")
        meta.setItem("duration", "durasi")
        meta.setItem("type", "tipe")
        meta.setItem("episode", "total episode")
        meta.setItem("rating")
        meta.setItem("score")

        sinopsis = content.find(class_="sinopsis")
        meta["sinopsis"] = sinopsis.div.text
        meta["image"] = content.img["src"]

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        if (dl := soup.find(class_="download")):
            if (batch := dl.find(class_="batch_content")):
                r = {}
                for p in batch.findAll("p"):
                    urls = {}
                    for a in p.findAll("a"):
                        urls[a.text] = a["href"]
                        a.decompose()
                    r[" ".join(p.text.split())] = urls
                result["Batch"] = r
            if (content := dl.find(class_="content")):
                for ul in content.findAll("ul"):
                    r = {}
                    for li in ul.findAll("li"):
                        urls = {}
                        for a in li.findAll("a"):
                            urls[a.text] = a["href"]
                        r[li.strong.text] = urls
                    result[ul.findPrevious("h3").text] = r
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}",
                               params={"s": query})
        soup = self.soup(raw)

        result = []
        for info in soup.findAll(class_="info-post"):
            if (a := info.a):
                result.append({
                    "id": self.getPath(a["href"]),
                    "title": a.text
                })
        return result



#!/usr/bin/python
from .thirdparty.exrex import generate as generate_strings
from .extractors import BaseExtractor
from .options import ArgumentParser
from .utils import parse_range
from urllib.parse import urlparse
from pkg_resources import parse_version
from . import __version__
import logging
import questionary
import sys
import re
import json
import colorama
import os
colorama.init()


logging.basicConfig(format=f"\x1b[K%(message)s", level=logging.INFO)
logging.getLogger("requests").setLevel(logging.WARNING)

'''
logging.info(f"""
  _____     ___  ____    _____    __
 |_   _|   |_  ||_  _|  / ___ `. /  |
   | |       | |_/ /   |_/___) | `| |
   | |   _   |  __'.    .'____.'  | |
  _| |__/ | _| |  \ \_ / /_____  _| |_
 |________||____||____||_______||_____| {__version__}
""")
'''


def title(text, rtn=False):
    r = f" [\x1b[92m{text}\x1b[0m]"
    if rtn:
        return r
    logging.info(r)


def _check_version():
    try:
        base = BaseExtractor()
        raw = base.session.get("https://pypi.org/project/lk21", timeout=2)
        soup = base.soup(raw)

        if (name := soup.find(class_="package-header__name")):
            version = name.text.split()[-1]
            if parse_version(__version__) < parse_version(version):
                return (
                    "\x1b[93m"
                    f"WARNING: Anda menggunakan lk21 versi {__version__}, sedangkan versi {version} telah tersedia.\n"
                    "Anda harus mempertimbangkan untuk mengupgrade melalui perintah 'python -m pip install --upgrade lk21'."
                    "\x1b[0m")
    except Exception:
        return


def recursive_choice(extractor, result):
    prevresult = None
    index = 0
    last = False

    #selected_items = []
    while True:
        # if last is True:
        #    otype = "checkbox"
        #    keys = [{
        #            "name": f"{key} (bypass)" if isinstance(
        #                value, str) and allDirectRules.search(value) else key,
        #            "checked": result.get(key) in selected_items
        #        } for key, value in result.items() if value]
        #    keys.extend([questionary.Separator(), {
        #        "name": "00. Kembali",
        #        "checked": False
        #    }])
        #    print (keys, selected_items)
        # else:
        #   otype = "list"

        keys = [
            f"{key} (bypass)" if isinstance(
                value, str) and Bypasser.allBypassPattern.search(value) else key
            for key, value in result.items() if value
        ]
        if prevresult and index > 0:
            keys.extend([questionary.Separator(), "00. Kembali"])

        key = extractor.choice(keys)

        # if isinstance(key, list):
        #    for k in key:
        #        if (v := result.get(k)):
        #            selected_items.append(v)

        if "00. Kembali" in key:
            last = False
            result = prevresult
            index -= 1
        else:
            prevresult = result
            index += 1
            result = result[key]

        if (index == 1 and isinstance(result, str)) or last is True:
            break

        if not all(isinstance(v, dict) for v in result.values()):
            last = True

    prefix = re.compile("^(?:" + r"|".join([extractor.host, "re\:"]) + ")")
    if prefix.search(result):
        id = extractor.getPath(prefix.sub("", result))
        logging.info(f"Mengekstrak link unduhan: {id}")
        result = extractor.extract(id).get("download", {})
        result = recursive_choice(extractor, result)
    return result


def main():
    global Bypasser

    extractors = {
        obj.__name__.lower(): obj for obj in BaseExtractor.__subclasses__() if obj
    }
    Bypasser = extractors.pop("bypass")(logging)
    Bypasser.run_as_module = False
    _version_msg = _check_version()

    parser = ArgumentParser(**{
        "extractors": extractors,
        "version_msg": _version_msg,
    })
    args = parser.parse_args()

    if args.version:
        sys.exit(__version__)

    if args.list_bypass:
        def add_color(x): return re.sub(
            r"\[[^]]+\]", lambda p: f"\x1b[33m{p[0]}\x1b[0m", x)
        for funcname, item in Bypasser.bypassPattern.items():
            logging.info(title(funcname.removeprefix("bypass_"), rtn=True))
            for rule in item["pattern"]:
                valid_url = re.sub(
                    r"\[.+?\][+*]\??|\\[a-zA-Z][+*]\??", "\[id\]", rule.pattern)
                valid_url = re.sub(r"\.[*+]\??", "\[any\]", valid_url)
                valid_url = re.sub(r"^https\??://",  "", valid_url)
                for s in generate_strings(valid_url):
                    logging.info(f"  • {add_color(s)}")
        sys.exit(0)
    elif args.bypass:
        result = Bypasser.bypass_url(args.bypass)
        logging.info(
            f"\n{result}\n")
        if args._exec:
            os.system(args._exec.format(result))
        sys.exit(0)

    if not args.query or (args._exec and "{}" not in args._exec):
        parser.print_help()
        sys.exit(0)

    extractor = extractors["layarkaca21"]
    for egn, kls in extractors.items():
        if args.__dict__[egn]:
            extractor = kls
            break

    extractor = extractor(logging, args)
    if "proxy" in getattr(extractor, "required", []) and not args.skip_proxy:
        if not args.proxy:
            parser.error(
                f"{extractor.__class__.__name__} required arguments --proxy, or skip using the --skip-proxy argument if already using vpn")
    extractor.run_as_module = args.json or args.json_dump

    if not extractor.tag and not args.debug:
        sys.exit(f"Module {extractor.__module__} belum bisa digunakan")
    query = " ".join(args.query)
    extractor.prepare()

    id = False
    nextPage = True
    Range = parse_range(args.page)
    netloc = urlparse(extractor.host).netloc
    try:
        page = Range.__next__()
        cache = {page: extractor.search(query, page=page)}
        while not id:
            print(
                f"Mencari {query!r} -> {netloc} halaman {page}")
            logging.info(
                f"Total item terkumpul: {sum(len(v) for v in cache.values())} item dari total {len(cache)} halaman")
            if not cache[page]:
                sys.exit("Tidak ditemukan")

            if len(cache[page]) == 1:
                response = f"1. " + cache[page][0]["title"]
            else:
                response = extractor.choice([
                    i['title'] for i in cache[page]] + [
                    questionary.Separator(), "00. Kembali", "01. Lanjut", "02. Keluar"], reset_counter=False)
            pgs = list(cache.keys())
            index = pgs.index(page)
            if response.endswith("Keluar"):
                break
            elif response.endswith("Kembali"):
                if extractor.counter > -1:
                    extractor.counter -= len(cache[page])
                print("\x1b[3A\x1b[K", end="")
                if index > 0 and len(pgs) > 1:
                    page = pgs[index - 1]
                    extractor.counter -= len(cache[page])
            elif response.endswith("Lanjut") and nextPage is True:
                if index >= len(pgs) - 1:
                    try:
                        ppage = Range.__next__()
                        if len(res := extractor.search(query, page=page)) > 0:
                            page = ppage
                            cache[page] = res
                        else:
                            extractor.counter -= len(cache[page])
                    except StopIteration:
                        nextPage = False
                else:
                    page = pgs[index + 1]
                if nextPage:
                    print("\x1b[3A\x1b[K", end="")
            else:
                for r in cache[page]:
                    if r.get("title") == re.sub(r"^\d+\. ", "", response):
                        id = r["id"]
                        break
        if id:
            logging.info(f"Mengekstrak link unduhan: {id}")
            result = extractor.extract(id)

            if args.json:
                sys.exit(f"\n{result}\n")
            elif args.json_dump:
                with open(args.json_dump, "w") as file:
                    file.write(json.dumps(result, indent=2, default=lambda o: o.store
                                          if isinstance(getattr(o, "store"), dict) else o.__dict__))
                sys.exit(
                    f"Menyimpan hasil unduhan kedalam file: {args.json_dump!r}")

            # cetak informasi jika ada
            extractor.info(f"\n [\x1b[92m{r.pop('title')}\x1b[0m]")
            for k, v in result.get("metadata", {}).items():
                extractor.info(
                    f"   {k}: {', '.join(filter(lambda x: x, v)) if isinstance(v, list) else v}")
            extractor.info("")

            result = result.get("download", {})
            dlurl = recursive_choice(extractor, result)
            url = Bypasser.bypass_url(dlurl)

            if args._exec:
                os.system(args._exec.format(url))
            else:
                logging.info("")
                title("Url Dipilih")
                logging.info(f"\n{url}\n")

    except Exception as e:
        logging.info(f"{e}")
        if args.debug:
            raise

    if _version_msg:
        logging.warning(_version_msg)



from . import BaseExtractor


class Meownime(BaseExtractor):
    """
    XXX: Semua isi konten sama dengan :Anibatch:
    """

    tag = "anime"
    host = "https://meownime.moe"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:h
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        meta["image"] = soup.find(class_="wp-post-image")["src"]
        if (content := soup.find(class_="entry-content")):
            meta.register(r"(?i){id} *: *(.+?)\n", content.text)

            meta.setItem("judul anime", "judul")
            meta.setItem("judul alternatif")
            meta.setItem("tipe anime", "tipe")
            meta.setItem("status anime", "status")
            meta.setItem("total episode")
            meta.setItem("musim rilis", "musim")
            meta.setItem("studio yang memproduksi", "studio")
            meta.setItem("genre")
            meta.setItem("durasi per episode", "durasi")
            meta.setItem("Skor di MyAnimeList", "score")

        if (h2 := soup.find("h2", text=self.re.compile(r"Sinopsis[^>]+"))):
            desc = []
            for p in h2.findAllNext("p"):
                if p.center:
                    break
                desc.append(p.text)
            meta["sinopsis"] = " ".join(desc)
        elif (ogDesc := soup.find(r"meta", property="og:description")):
            content = self.re.sub(r"\[[^]]+?]\s*$", "[^>]+", ogDesc["content"])
            if (fullDesc := soup.find(text=self.re.compile(content))):
                meta["sinopsis"] = fullDesc

        if (h2 := soup.find("h2", text="Main Character")):
            meta["karakter"] = [
                figure.text for figure in h2.findAllNext("figure")]

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        dlx = soup.find(class_="dlx")
        result = {}

        for table in dlx.findAll("table"):
            if not table.strong:
                continue

            if (h4 := table.findPrevious("h4", style="text-align: center")):
                eps = h4.text
            elif (prevTable := table.findPrevious("table")):
                eps = prevTable.text

            d = {}
            res = None
            for tr in table.findAll("tr"):
                if not tr.a:
                    res = tr.text
                    d[res] = {}
                    continue
                for a in tr.findAll("a"):
                    d[res][a.text] = a["href"]
            result[eps] = d
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query})
        soup = self.soup(raw)

        result = []
        for artikel in soup.findAll("article"):
            a = artikel.find("a")
            if not a.img:
                continue
            result.append({
                "title": a["title"],
                "id": self.getPath(a["href"])
            })
        return result



from . import BaseExtractor

class Bbcsjav(BaseExtractor):
    tag = "JAV"
    host = "https://bbcsjav.com"
    required = ["proxy"]

    def getCode(self, text):
        if (code := self.re.search(r"([A-Z]+-[0-9]+)", text)):
            return code.group()

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        meta["title"] = "-".join(soup.title.text.split("-")[:-1]).strip(". ")
        meta["image"] = soup.find("img", class_="wp-post-image")["src"]
        meta["code"] = self.getCode(soup.title.text)

        info = soup.find(class_="info_vids")
        aliases = {
            "release": "rilis",
            "duration": "durasi",
            "genres": "genre",
            "quality": "kualitas"
        }
        for p in info.findAll("p"):
            if ":" in p.text:
                k, v = self.re.split("\s*:\s*", p.text)
                k = k.lower()
                k = aliases.get(k, k)
                if "," in v:
                    v = self.re.split("\s*,\s*", v)
                meta[k] = v

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}

        kontent = soup.find(class_="kontent")
        for h3 in kontent.findAll("h3"):
            items = {}
            for tr in h3.findNext("div").tbody.findAll("tr"):
                subitem = {}
                quality = None
                for td in tr.findAll("td"):
                    if not td.a:
                        quality = td.text
                    else:
                        for a in td.findAll("a"):
                            subitem[a.text] = a["href"]
                items[quality] = subitem
            result[h3.text] = items
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}",
           params={"s": query})
        soup = self.soup(raw)

        result = []
        for post_id in soup.findAll(id="post-ID"):
            a = post_id.a
            title = post_id.find(class_="title")
            result.append({
                "id": self.getPath(a["href"]),
                "title": title.text.strip(),
                "tag": [tag.text for tag in post_id.findAll(rel="tag")],
                "code": self.getCode(title.text)
            })
        return result



from . import BaseExtractor


class Riie(BaseExtractor):
    tag = "anime"
    host = "https://riie.jp"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'

        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        meta["image"] = soup.find(class_="dc-thumb").img["src"]
        meta["judul"] = soup.find(class_="dc-title").text
        meta["sinopsis"] = soup.find(class_="dci-desc").text
        meta["rating"] = soup.find(id="vote_percent").text

        def setItem(text, alias=None):
            td = soup.find("td", text=text)
            v = td.findNext("td")
            if v.a:
                v = [a.text for a in v.findAll("a")]
            else:
                v = v.text
            meta.add(text, v)

        setItem("Judul Alternatif")
        setItem("Status")
        setItem("Genre")

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if (eps := soup.findAll(id="episodes-list")):
            lst = {}
            for ep in eps:
                if (items := ep.findAll("li", class_="ep-item")):
                    title = ep.find(class_="gh-title").text

                    raw = {}
                    for item in items:
                        a = item.find("a")
                        if "download" in title.lower():
                            raw[a.text] = a["href"]
                        else:
                            raw[a.text] = "re:" + self.getPath(a["href"])
                    lst[title] = raw
            return lst

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/search/{page}/{query}")
        soup = self.soup(raw)

        res = []
        if (result := soup.find(class_="filter-result")):
            for ul in result.findAll("ul"):
                for li in ul.findAll("li"):
                    a = li.find(class_="item-title").a
                    r = {
                        "id": self.getPath(a["href"]),
                        "title": a.text
                    }
                    for kl in ("tv-type", "gr-eps", "gr-type", "gr-sub"):
                        if (it := li.find(class_=kl)):
                            r[kl] = it.text
                    res.append(r)
        return res



from . import BaseExtractor


class Otakudesu(BaseExtractor):
    tag = "anime"
    host = "https://otakudesu.moe"

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if (eps := soup.findAll(class_="episodelist")[1:]):
            ch = {}
            for li in eps[0].findAll("li"):
                ch[li.a.text] = li.a["href"]
            return ch
        else:
            dls = {}
            for dl in soup.findAll("div", class_="download"):
                title = [h4.text for h4 in dl.findAll("h4")]
                for n, ul in enumerate(dl.findAll("ul")):
                    if len(title) == 1:
                        n = 0
                    n = title[n]
                    if not dls.get(n):
                        dls[n] = {}
                    for li in ul.findAll("li"):
                        dls[n][f"{li.strong.text}/{li.i.text}"] = {
                            a.text: a["href"] for a in li.findAll("a")
                        }
            return dls

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(self.host, params={
            "s": query, "post_type": "Anime"})
        soup = self.soup(raw)

        result = []
        if (ul := soup.find("ul", class_="chivsrc")):
            for li in ul.findAll("li"):
                result.append({
                    "title": li.a.text,
                    "id": self.getPath(li.a["title"])
                })
        return result



from . import BaseExtractor


class Oploverz(BaseExtractor):
    host = "https://www.oploverz.in"
    tag = "anime"
    required = ["proxy"]

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        if (ep_list := soup.find(class_="episodelist")):
            for li in ep_list.findAll("li"):
                a = li.find("a")
                result[a.text.strip()] = a["href"]
            return result

        elif (all_eps := soup.find(class_="btn-full-eps")):
            result[all_eps.text] = "re:" + self.getPath(all_eps["href"])

        title = [
            a for a in self.re.findall(r"(?s)strong>([^>]+?)</strong", str(soup))
            if self.re.search(r"\([^)]+", a)
        ]

        for ttl, ddl in zip(title, soup.findAll(class_="soraddl")):
            d = {}
            for p in ddl.findAll("p"):
                if (t1 := p.find(class_="sorattl")):
                    d1 = {}
                    for a in p.findAll("a"):
                        d1[a.text] = a["href"]
                    d[t1.text] = d1
            result[ttl] = d
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(f"{self.host}/page/{page}/",
                               params={"s": query})
        soup = self.soup(raw)

        result = []
        for dtl in soup.findAll("div", class_="dtl"):
            a = dtl.find("a")
            result.append({
                "title": a.text,
                "id": self.getPath(a["href"]),
            })
        for bs in soup.findAll("article", class_="bs"):
            result.append({
                "id": self.getPath(bs.a["href"]),
                "title": bs.find(itemprop="headline").text
            })
        return result



from . import BaseExtractor


class Wibudesu(BaseExtractor):
    host = "https://wibudesu.com"
    tag = "anime"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        content = soup.find("div", class_="lexot")
        meta = self.MetaSet()

        meta.register(None, content.text)
        meta.setItem("judul")
        meta.setItem("native title", "judul alternatif")
        meta.setItem("also known as", "judul alternatif")
        meta.setItem("genres", "genre")
        meta.setItem("movie")
        meta.setItem("country", "negara")
        meta.setItem("release date", "rilis")
        meta.setItem("duration", "durasi")
        meta.setItem("score")

        meta["image"] = content.img["data-lazy-src"]
        meta["judul"] = self.re.split(
            "(?i)(?:bd )?(?:batch )?subtitle", soup.title.text)[0]

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        lexot = soup.find(class_="lexot")

        result = {}
        for p in lexot.findAll("p")[1:]:
            if (links := p.findAll("a")):
                title = p.strong.text
                d = {}
                for a in links:
                    d[a.text] = a["href"]
                result[title] = d
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query, "post_type": "post"})
        soup = self.soup(raw)

        r = []
        for detpost in soup.findAll(class_="detpost"):
            a = detpost.find("a")
            result = {
                "title": a["title"],
                "id": self.getPath(a["href"])
            }

            if (morec := detpost.find(class_="morec")):
                result["genre"] = [a.text for a in morec.findAll("a")]

            r.append(result)
        return r



from ..thirdparty import exrex
from . import BaseExtractor
from urllib.parse import urlparse
from collections import defaultdict
import re
import inspect
import math


class Bypass(BaseExtractor):
    bypassPattern = defaultdict(lambda: defaultdict(set))
    allBypassPattern = None

    def bypass_ouo(self, url):
        """
        regex: https?://ouo\.(?:press|io)/[^>]+
        """

        def getData(form):
            if not form:
                return None, None
            data = {}
            for input in form.findAll("input", {"name": re.compile(r"token$")}):
                k = input.get("id") or input.get("name")
                data[k] = input["value"]
            return form["action"], data

        raw = self.session.get(url)
        soup = self.soup(raw)
        action, data = getData(soup.form)

        for _ in range(2):
            self.report_bypass(action)
            raw = self.session.post(action, data=data)
            soup = self.soup(raw)
            action, data = getData(soup.form)

        return raw.url

    def bypass_anonfiles(self, url):
        """
        regex: https?://anonfiles\.com/[^>]+
        """

        raw = self.session.get(url)
        soup = self.soup(raw)

        if (dlurl := soup.find(id="download-url")):
            return dlurl["href"]

    def bypass_letsupload(self, url):
        """
        regex: https?://letsupload\.[ic]o/[0-9A-Za-z]+(?:/|\?pt=)[^>]+
        """

        raw = self.session.get(url)
        if (fileId := re.search(r"(?i)showFileInformation\((\d+)\)", raw.text)):
            return self.bypass_redirect('https://letsupload.io/account/direct_download/' + fileId.group(1))
        if (nextUrl := re.search(r"window.location += ['\"]([^\"']+)", raw.text)):
            return nextUrl.group(1)

    def __bypass_streamtape(self, url):
        """
        regex: https?://streamtape\.com/v/[^/]+/[^>]+
        """

        raw = self.session.get(url)
        soup = self.soup(raw)

        if (videolink := soup.find(id="videolink")):
            nexturl = "https:" + videolink.text
            self.report_bypass(nexturl)
            if (redirect := self.bypass_redirect(nexturl)):
                return redirect

    def bypass_sbembed(self, url):
        """
        regex: https?://sbembed\.com/[^>]+\.html
        regex: https?://streamsb\.net/[^>]+\.html
        """

        raw = self.session.get(url)
        soup = self.soup(raw)

        result = {}
        for a in soup.findAll("a", onclick=re.compile(r"^download_video[^>]+")):
           data = dict(zip(["id", "mode", "hash"], re.findall(r"[\"']([^\"']+)[\"']", a["onclick"])))
           data["op"] = "download_orig"

           raw = self.session.get("https://sbembed.com/dl", params=data)
           soup = self.soup(raw)

           if (direct := soup.find("a", text=re.compile("(?i)^direct"))):
               result[a.text] = direct["href"]
        return result

    def bypass_filesIm(self, url):
        """
        regex: https?://(?:files\.im|racaty\.net|hxfile\.co)/[^>]+
        """

        headers = {
            'content-type': 'application/x-www-form-urlencoded',
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.152 Safari/537.36',
        }

        data = {
            'op': 'download2',
            'id': self.getPath(url),
            'rand': '',
            'referer': '',
            'method_free': '',
            'method_premium': '',
        }

        response = self.session.post(url, headers=headers, data=data)
        soup = self.soup(response)

        if (btn := soup.find(class_="btn btn-dow")):
            return btn["href"]
        if (unique := soup.find(id="uniqueExpirylink")):
            return unique["href"]

    def bypass_redirect(self, url):
        """
        regex: https?://bit\.ly/[^>]+
        regex: https?://(?:link\.zonawibu\.cc/redirect\.php\?go|player\.zafkiel\.net/blogger\.php\?yuzu)\=[^>]+
        """
        head = self.session.head(url)
        return head.headers.get("Location", url)

    def bypass_linkpoi(self, url):
        """
        regex: https?://(?:www\.)?uservideo\.xyz/file/[^>]+
        regex: https?://linkpoi\.me/[^>]+
        """

        raw = self.session.get(url)
        soup = self.soup(raw)

        if (a := soup.find("a", class_="btn-primary")):
            return a["href"]

    def bypass_mediafire(self, url):
        """
        regex: https?://(?:www\.)?mediafire\.com/file/[^>]+(?:/file)?
        """

        raw = self.session.get(url)
        soup = self.soup(raw)

        if (dl := soup.find(id="downloadButton")):
            return dl["href"]

    def __bypass_zippyshare(self, url):
        """
        regex: https?://www\d+\.zippyshare\.com/v/[^/]+/file\.html
        """

        raw = self.session.get(url)
        soup = self.soup(raw)

        if (script := soup.find("script", text=re.compile("(?si)\s*var a = \d+;"))):
            sc = str(script)
            var = re.findall(r"var [ab] = (\d+)", sc)
            omg = re.findall(r"\.omg (!?=) [\"']([^\"']+)", sc)
            file = re.findall(r'"(/[^"]+)', sc)
            if var and omg:
                a, b = var
                if eval(f"{omg[0][1]!r} {omg[1][0]} {omg[1][1]!r}") or 1:
                    a = math.ceil(int(a) // 3)
                else:
                    a = math.floor(int(a) // 3)
                divider = int(re.findall(f"(\d+)%b", sc)[0])
                return re.search(r"(^https://www\d+.zippyshare.com)", raw.url).group(1) + \
                    "".join([
                        file[0],
                        str(a + (divider % int(b))),
                        file[1]
                    ])

    def bypass_fembed(self, url):
        """
        regex: https?://(?:www\.naniplay|naniplay)(?:\.nanime\.(?:in|biz)|\.com)/file/[^>]+
        regex: https?://layarkacaxxi\.icu/f/[^>]+
        """

        raw = self.session.get(url)
        api = re.search(r"(/api/source/[^\"']+)", raw.text)
        if api is not None:
            result = {}
            raw = self.session.post(
                "https://layarkacaxxi.icu" + api.group(1)).json()
            for d in raw["data"]:
                f = d["file"]
                direct = self.bypass_redirect(f)
                result[f"{d['label']}/{d['type']}"] = direct
            return result

    def report_bypass(self, url):
        if self.logger:
            self.logger.info(f"Bypass link {urlparse(url).netloc}: {url}")

    def bypass_url(self, url):
        """
        Bypass url

        Args:
              url: type 'str'
        """

        patterns = {k: v['pattern'] for k, v in self.bypassPattern.items()}

        previous = url
        stop = False
        while not stop:
            _break = False
            for fname, pattern in patterns.items():
                if isinstance(url, dict) or _break:
                    break

                if any(_re.search(url) for _re in pattern):
                    self.report_bypass(url)

                    func = getattr(self, fname)
                    try:
                        result = func(url)
                    except Exception:
                        result = None

                    if not result:
                        stop = True
                        break

                    if self.run_as_module or not isinstance(result, dict):
                        url = result
                    else:
                        key = self.choice(result.keys())
                        url = result[key]
                    if url == previous:
                        stop = not stop
                    else:
                        previous = url
                    _break = True
            if not _break:
                break

        if isinstance(url, str):
            return re.sub(r" ", "%20", url)
        else:
            return url


# lazy method
allBypassPattern = []
pattern = re.compile(r"regex *: *(.+)(?:\n|$)")
for key, value in inspect.getmembers(Bypass):
    if key.startswith("bypass_") and key != "bypass_url":
        for urlPattern in pattern.findall(value.__doc__ or ""):
            Bypass.bypassPattern[key]["pattern"].add(
                re.compile(urlPattern)
            )
            allBypassPattern.append(urlPattern)
        if (allPattern := Bypass.bypassPattern.get(key)):
            for rule in allPattern["pattern"]:
                valid_regex = re.sub(
                    r"\[.+?\][+*]\??|\\[a-zA-Z][+*]\??", "\[id\]", rule.pattern)
                valid_regex = re.sub(r"\.[*+]\??", "\[any\]", valid_regex)
                for url in exrex.generate(valid_regex):
                    Bypass.bypassPattern[key]["support"].add(
                        urlparse(url).netloc.removeprefix("www."))

Bypass.allBypassPattern = re.compile(r"|".join(allBypassPattern))



from . import BaseExtractor


class Samehadaku(BaseExtractor):
    tag = "anime"
    host = "https://samehadaku.vip"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        alias = {
            'Japanese': "judul alternatif",
            'English': "judul alternatif",
            'Synonyms': "judul alternatif",
            'Season': 'musim',
            'Producers': 'produser',
            'type': 'tipe',
            'Duration': 'durasi',
        }
        meta = self.MetaSet()
        meta["image"] = soup.find("img", class_="anmsa")["src"]
        meta["judul"] = self.re.split(
            "(?i)(?:bd )?(?:batch )?subtitle", soup.title.text)[0]
        meta["sinopsis"] = soup.find(class_="desc").text

        content = soup.find(class_="spe")
        for span in content.findAll("span"):
            k = span.b.text.rstrip(":")
            span.b.decompose()
            k = alias.get(k, k)

            if span.a:
                v = [a.text for a in span.findAll("a")]
            else:
                v = span.text
            meta.add(k, v, split=k not in ["Rilis"])

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if id.startswith("anime/"):
            ch = {}
            if (listeps := soup.findAll(class_="epsleft")):
                for li in listeps:
                    a = li.find("a")
                    ch[a.text] = self.getPath(a["href"])
            if (batch := soup.find(class_="listbatch")):
                ch[batch.text] = self.getPath(batch.a["href"])
            return ch

        result = {}
        for dl in soup.findAll(class_="download-eps"):
            d = {}
            for li in dl.findAll("li"):
                item = {}
                for a in li.findAll("a"):
                    item[a.text] = a["href"]
                d[li.strong.text] = item
            result[dl.p.text] = d
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query})
        soup = self.soup(raw)

        res = []
        for article in soup.findAll("article", class_="animpost"):
            result = {
                "id": self.getPath(article.find("a")["href"])
            }

            for k in ("score", "title", "type", "genres"):
                if (v := article.find(class_=k)):
                    name = " ".join(v["class"])
                    if (aa := v.findAll("a")):
                        result[name] = [a.text for a in aa]
                    elif v.text:
                        result[name] = v.text
            res.append(result)
        return res



from . import BaseExtractor


class Anitoki(BaseExtractor):
    tag = "anime"
    host = "https://www.anitoki.com"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        if (sinop := soup.find(text=self.re.compile(r"Sinopsis[^>]+"))):
            meta["sinopsis"] = " ".join(
                p.text for p in sinop.findAllNext("p", style="text-align: justify;") if not p.span
            )

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        for dl in soup.findAll(class_="dlbod"):
            for smokeurl in dl.findAll(class_="smokeurl"):
                smokettl = smokeurl.findPrevious(class_="smokettl").text
                if not result.get(smokettl):
                    result[smokettl] = {}

                r = {}
                for a in smokeurl.findAll("a"):
                    r[a.text] = a["href"]
                if smokeurl.strong:
                    result[smokettl][smokeurl.strong.text] = r
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}",
                               params={"s": query})
        soup = self.soup(raw)

        result = []
        for kover in soup.findAll("div", class_="kover"):
            a = kover.a
            result.append({
                "id": self.getPath(a["href"]),
                "title": a["title"]
            })
        return result



from . import BaseExtractor


class Anikyojin(BaseExtractor):
    host = "https://anikyojin.net"
    tag = "anime"
    required = ["proxy"]

    def prepare(self):
        if self.args:
            self.setProxies(self.args.proxy)

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        meta["image"] = soup.find(class_="wp-post-image")["src"]
        if (infox := soup.find(class_="infox")):
            meta.register(r"(?i){id}\s*:\s*([^>]+?) *?\n", infox.text)

            meta.setItem("judul")
            meta.setItem("judul lain", "judul alternatif")
            meta.setItem("type", "tipe")
            meta.setItem("episode", "total episode")
            meta.setItem("duration", "durasi")
            meta.setItem("category", "genre")
            meta.setItem("aried", "tayang")
            meta.setItem("studio")
            meta.setItem("rating")
            meta.setItem("score", split=False)

        if (sinop := soup.find(class_="sinop")):
            meta["sinopsis"] = sinop.p.text

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        for dc in soup.findAll(class_="downloadcloud"):
            reso = {}
            for li in dc.findAll("li"):
                d = {}
                for a in li.findAll("a"):
                    d[a.text] = a["href"]
                reso[li.strong.text] = d
            result[dc.h2.text] = reso
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query, "post_type": "post"})
        soup = self.soup(raw)

        r = []
        for article in soup.findAll(class_="artikel"):
            a = article.h2.find("a")

            result = {
                "title": a.text,
                "id": self.getPath(a["href"])
            }

            for li in article.find(class_="info").findAll("li"):
                k, v = self.re.split(r"\s*:\s*", li.text)
                if "," in v:
                    v = self.re.split(r"\s*,\s*", v)
                result[k] = v

            r.append(result)
        return r



from . import BaseExtractor


class Animeindo(BaseExtractor):
    tag = "anime"
    host = "https://animeindo.asia"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        meta["judul"] = soup.find(class_="entry-title").text

        if (infox := soup.find(class_="cat_box_desc")):
            meta.register(r"(?si){id}\s*:\s*([^>]+?)\s*?\n", infox.text)

            meta.setItem("status")
            meta.setItem("type", "tipe")
            meta.setItem("score")
            meta.setItem("aired", "tayang")
            meta.setItem("duration", "durasi")
            meta.setItem("genres", "genre")
            meta.setItem("season", "musim")

        if (sinops := soup.find(class_="sinops")):
            meta["sinopsis"] = sinops.div.text

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        if (eps := soup.findAll(class_="episode_list")):
            d = {}
            for ep in eps:
                d[ep.a.text.split(" -")[0]] = "re:" + \
                    self.getPath(ep.a["href"])
            return d

        formats = self.re.findall(r"(Format .+?):", str(soup))
        result = {fm: {} for fm in formats}
        for p in soup.findAll("p", class_="has-text-align-center"):
            if (aa := p.findAll("a")):
                info = [self.re.split(r"\s*:", i)[0] for i in
                        self.re.findall(r">\s*(\d+p(?:\s+.+?)?)\s*:", str(p))]
                if len(info) == 1:
                    d = {info[0]: {
                        a.text: a["href"] for a in aa
                    }}
                else:
                    formats.pop(0)
                    d = {}
                    for sub, dom in self.re.findall(r"(\d+p.+?):.+?((?:<a.+?>.+?</a>(?:\s*\|\s*)?)+)", str(p)):
                        aa = self.soup(dom).findAll("a")
                        d[sub] = {
                            a.text: a["href"] for a in aa
                        }
                result[formats[0]] = d
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query})
        soup = self.soup(raw)

        result = []
        for animposx in soup.findAll(class_="animposx"):
            a = animposx.find("a")
            result.append({
                "title": a["title"],
                "id": self.getPath(a["href"])
            })
        return result



from urllib.parse import urlparse
from http.cookiejar import LWPCookieJar
import requests
import requests_cache
import bs4
import re
import questionary
import sys
import glob
import os
import importlib

from ..utils import MetaSet


class BaseExtractorError(Exception):
    pass


requests_cache.install_cache(cache_name=os.path.expanduser(
    '~/.lk21-requests-cache'), backend='sqlite', expire_after=90)


class BaseExtractor:
    def __init__(self, logger=None, args=None):
        """
        Induk dari semua 'extractor'

        Args:
              logger: logger instance
              args: 'argparse.Namespace'
        """

        self.session = self._build_session()
        self.re = re
        self.logger = logger
        self.args = args
        self.counter = 0
        self.run_as_module = True

        self.MetaSet = MetaSet

    def _build_session(self) -> requests.Session:
        """
        Buat session baru
        """

        session = requests.Session()
        session.headers[
            "User-Agent"] = "Mozilla/5.0 (Linux; Android 7.0; 5060 Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/58.0.3029.83 Mobile Safari/537.36"
        session.cookies = LWPCookieJar()
        return session

    def setProxies(self, proxy: str) -> None:
        if not proxy:
            return
        if self.logger:
            self.logger.info(f"Menyetel proxy: {proxy}")
        self.session.proxies = {
            "http": f"http://{proxy}",
            "https": f"https://{proxy}",
        }

    def prepare(self):
        """
        ubah :self: sebelum membuat permintaan
        """

        pass

    def dict_to_list(self, d: dict) -> list:
        """
        undocumented
        """

        prefix = self.re.compile(r"^(?:re\:|{})".format(self.host))
        if not self.run_as_module:
            return d
        list_ = []
        for k, v in d.items():
            if isinstance(v, dict):
                v = self.dict_to_list(v)
            elif isinstance(v, str) and prefix.search(v):
                id = prefix.sub("", v).strip("/")
                if self.args and (self.args.json or self.args.json_dump):
                    if self.logger:
                        self.logger.info(f"Mengekstrak link unduhan: {id}")
                    data = self.extract_data(id)
                    if len(data) == 1:
                        data = list(data.values())[0]
                    v = self.dict_to_list(data)
                else:
                    v = self.host + "/" + id
            list_.append({
                "key": k.strip(),
                "value": v
            })
        return list_

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        return {}

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        return {}

    def filterNonetypeDict(self, items: list, keys: tuple) -> list:
        """
        undocumented
        """

        filtered = []
        for item in items:
            if all(item.get(k) not in [None, ""] for k in keys):
                filtered.append(item)
        return filtered

    def extract(self, id: any) -> dict:
        """
        Args:
            id: type 'dict' or 'str'
        """

        if isinstance(id, dict):
            if not id.get("id"):
                raise BaseExtractorError("You must provide a `id` value")
            id = id["id"]

        result = {
            "extractor": self.__class__.__name__,
            "tag": self.tag,
            "url": f"{self.host}/{id}",
            "host": self.host,
            "id": id,
        }
        meta = self.extract_meta(id)
        result["metadata"] = meta.store if isinstance(meta, self.MetaSet) else meta
        result["download"] = self.dict_to_list(
            self.extract_data(id))

        return result

    def soup(self, raw: str) -> bs4.BeautifulSoup:
        """
        Ubah 'requests.Response' atau 'str' menjadi 'bs4.BeautifulSoup'

        Args:
              raw: type 'requests.Response' atau 'str'
        """

        text = raw.text if hasattr(raw, "text") else raw
        return bs4.BeautifulSoup(text, "html.parser")

    def getPath(self, url: str) -> str:
        """
        undocumented
        """

        return urlparse(url).path.strip("/")

    def _write(self, soup: bs4.BeautifulSoup, file: str = "x.html") -> None:
        """
        Args:
              soup: type 'requests.Response' atau 'str'
              file: type 'str'
        """

        if hasattr(soup, "prettify"):
            soup = soup.prettify()
        with open(file, "wb") as f:
            f.write(soup.encode())

    def _reformat(self, raw: str, add_counter: bool = True) -> str:
        """
        Reformat text
        """

        if not isinstance(raw, str) or re.match(r"^\d+\.", raw):
            return raw
        else:
            self.counter += 1
            raw = re.sub(r"^\s+|\s+$", "", raw)
            if add_counter:
                return f"{self.counter}. {raw}"
            else:
                return raw

    def info(self, *args, **kwargs):
        """
        undocumented
        """

        if self.logger and self.args and self.args.info:
            self.logger.info(*args, **kwargs)

    def choice(self, choices, msg=None, reset_counter=True, otype="list"):
        """
        undocumented
        """

        try:
            if reset_counter:
                self.counter = 0
            choices = list(choices)
            if len(choices) == 0:
                sys.exit("Pilihan kosong")

            if len(choices) == 1:
                return choices[0]

            if otype == "list":
                nch = {
                    self._reformat(v): v for v in choices}
                prompt = questionary.select(
                    message=msg or "Pilih:",
                    choices=nch.keys()
                )
            elif otype == "checkbox":
                choices = [
                    {"name": self._reformat(k["name"], add_counter=False),
                     "checked": k["checked"]} if isinstance(k, dict) else k
                    for k in choices]
                prompt = questionary.checkbox(
                    message=msg or "Pilih:",
                    choices=choices,
                    validate=lambda x: "Pilihan tidak boleh kosong" if len(
                        x) == 0 else True
                )
            else:
                raise TypeError(f"{otype!r} is not allowed")

            if (output := prompt.ask()):
                valid_id = re.compile(r" \(bypass\)$")
                if isinstance(output, str):
                    return valid_id.sub("", nch[output])
                return [valid_id.sub("", o) for o in output]
        except Exception:
            sys.exit(1)


basedir = os.path.dirname(__file__)
for file in glob.glob(f"{basedir}/*.py"):
    filename = os.path.basename(file)[:-3]
    if not filename.startswith("__"):
        importlib.import_module(f"lk21.extractors.{filename}")
del basedir, file, filename



from . import BaseExtractor


class Drivenime(BaseExtractor):
    tag = "anime"
    host = "https://drivenime.com"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        content = soup.find(class_="post-single-content")
        meta.register(None, content.text)
        meta["image"] = soup.find("img", class_="alignnone")["src"]
        meta["judul"] = soup.title.text.split("Subtitle")[0]
        meta.setItem("romaji", "judul alternatif")
        meta.setItem("japanese", "judul alternatif")
        meta.setItem("type", "tipe")
        meta.setItem("episodes", "total episode")
        meta.setItem("status")
        meta.setItem("start", split=False)
        meta.setItem("end", split=False)
        meta.setItem("season", "musim")
        meta.setItem("producers", "produser")
        meta.setItem("main studio", "studio")
        meta.setItem("duration", "durasi")
        meta.setItem("genres", "genre")

        meta["tayang"] = meta.pop(
            "start", "?", force=True) + " - " + meta.pop("end", "?", force=True)

        if (sinop := content.find("h2", text="Sinopsis")):
            s = []
            for p in sinop.findAllNext(["p", "h2"]):
                if p.name != "p":
                    break
                s.append(p.text)
            meta["sinopsis"] = " ".join(s)
        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        dl = soup.find(class_="post-single-content")
        for p in dl.findAll("p"):
            if (p.find("a")) and "download" in p.text.lower():
                break
        title = self.re.findall(r"(?s)a>\s*(\[[^>]+?\])\s*<", str(p))
        return {
            ttl: a["href"] for ttl, a in zip(title, p.findAll("a"))
        }

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'

        Returns:
              list: daftar item dalam bentuk 'dict'
        """

        raw = self.session.get(f"{self.host}/page/{page}", params={
            "s": query})
        soup = self.soup(raw)

        result = []
        for post in soup.findAll(class_="post"):
            a = post.find("a")
            r = {
                "title": a["title"],
                "id": self.getPath(a["href"])
            }

            if (genre := post.find(class_="theauthor")):
                r["genre"] = [a.text for a in genre.findAll("a")]

            result.append(r)
        return result



import re
from collections import UserDict


class MetaSet(UserDict):
    def __init__(self, *args, **kwargs):
        self.store = dict()
        self.update(dict(*args, **kwargs))  # use the free update to set keys

        self._pattern = r"(?i){id} *: *(.+?)\n"
        self._content = ""

    def __getitem__(self, key):
        return self.store[key]

    def __setitem__(self, key, value):
        self.store[key] = value.strip() if isinstance(value, str) else value

    def __delitem__(self, key):
        del self.store[key]

    def __iter__(self):
        return iter(self.store)

    def __len__(self):
        return len(self.store)

    def __repr__(self):
        return repr(self.store)

    def register(self, pattern: str, content: str) -> None:
        """
        Menyetel pola Regex
        """

        if pattern:
            self._pattern = pattern
        if content:
            self._content = content

    def add(self, key: any, value: any, split: bool = True) -> None:
        key = re.sub(r" ", "_", key).lower()
        if isinstance(value, str):
            value = re.split(r"\s*,\s+", value
                             ) if split else [value]
        if not value:
            return
        if len(value) == 1:
            value = value[0].strip()
        if (pre := self.store.get(key)):
            if not isinstance(pre, list):
                self.store[key] = [pre]
            self.store[key].extend(
                [value] if not isinstance(value, list) else value)
        else:
            self.store[key] = value

    def setItem(self, id: str, key: str = None, split: bool = True) -> None:
        """
        Tambah item jika ditemukan
        """

        if (value := re.search(self._pattern.format(id=id), self._content)):
            self.add(key or id, value.group(1), split)

    def pop(self, key, default=None, force=False):
        if not self.store.get(key):
            return default
        else:
            return self.store.pop(key) or default


def parse_range(raw):
    assert re.match(r"[\s\d:]+", raw), f"invalid syntax: {raw!r}"
    if "," in raw:
        for rg in re.split(r"\s*,\s*", raw):
            if ":" not in rg:
                yield rg
            else:
                assert len(re.findall(r":", raw)
                           ) <= 1, f"invalid syntax: {raw!r}"
                yield from parse_range(rg)
    else:
        spl = re.split(r"\s*:\s*", raw)
        if not spl[0]:
            spl[0] = "1"

        start, end = map(lambda x: int(x) if x else None, spl)
        if end:
            assert start < end, "angka pertama tidak boleh lebih dari angka kedua"
            yield from range(start, end + 1)
        else:
            while True:
                yield start
                start += 1



from . import BaseExtractor


class Layarkaca21(BaseExtractor):
    tag = "movie"
    host = "http://149.56.24.226/"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        img = soup.find(class_="img-thumbnail")

        meta = self.MetaSet()
        meta["judul"] = img["alt"]
        meta["image"] = "https:" + img["src"]

        content = soup.find(class_="content")
        for div in content.findAll("div"):
            if (k := div.h2) and (k := k.text) and k not in ["Oleh", "Diunggah"]:
                value = ", ".join(h3.text for h3 in div.findAll("h3"))
                meta.add(k, value, split=k not in ["Imdb", "Diterbitkan"])
        if (block := soup.find("blockquote")):
            block.strong.decompose()
            block.span.decompose()

            meta["sinopsis"] = block.text

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'

        Returns:
              dict: hasil 'scrape' halaman web
        """

        raw = self.session.post("http://dl.sharemydrive.xyz/verifying.php",
                                headers={
                                    "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
                                    "Accept": "*/*",
                                    "X-Requested-With": "XMLHttpRequest"
                                },
                                params={"slug": id},
                                data={"slug": id}
                                )
        soup = self.soup(raw)
        tb = soup.find("tbody")

        result = {}
        for tr in tb.findAll("tr"):
            title = tr.find("strong").text
            result[title] = {}
            for td in tr.findAll("td")[1:]:
                if (a := td.a):
                    result[title][a["class"]
                                  [-1].split("-")[-1]] = a["href"]
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(self.host,
                               params={"s": query})

        soup = self.soup(raw)

        r = []
        for item in soup.findAll(class_="search-item"):
            a = item.a
            extra = {"genre": [], "star": [], "country": [],
                     "size": [""], "quality": [""], "year": [""]}
            for tag in item.find(class_="cat-links").findAll("a"):
                name, it = self.re.findall(r"/([^/]+)/([^/]+)", str(tag))[0]
                extra[name].insert(0, it)

            for p in filter(lambda x: x.strong is not None, item.findAll("p")):
                np, vl = self.re.findall(
                    r"^([^:]+):\s+(.+)", p.text.strip())[0]
                np = "star" if np == "Bintang" else "director" if np == "Sutradara" else np
                extra[np] = self.re.split(r"\s*,\s*", vl) if "," in vl else vl

            extra["id"] = self.re.search(
                r"\w/([^/]+)", a["href"]).group(1)
            result = {
                "title": (item.find("h2").text or a.img["alt"]).strip(),
            }
            result.update(extra)
            r.append(result)
        return r



from .thirdparty.exrex import getone as regex_to_string
from shutil import get_terminal_size
from .extractors.bypasser import Bypass
from urllib.parse import urlparse
import argparse
import re


def ipPortType(arg_value, pat=re.compile(r"^(?:\d{1,3}\.?){4}:\d+$")):
    if not pat.match(arg_value):
        raise argparse.ArgumentTypeError("Invalid proxy format")
    return arg_value


def ArgumentParser(**kwargs):
    extractors = kwargs.pop("extractors")
    version_msg = kwargs.pop("version_msg")

    parser = argparse.ArgumentParser(
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(
            prog, max_help_position=get_terminal_size().lines),
        epilog=version_msg)

    parser.add_argument("query", metavar="query",
                        nargs="*", help="kueri, judul, kata kunci")
    parser.add_argument("--version", action="store_true",
                        help="show version and exit")
    parser.add_argument("-d", "--debug", action="store_true",
                        help=argparse.SUPPRESS)
    parser.add_argument("-p", metavar="page", dest="page",
                        help=("halaman situs, contoh penggunaan:\n"
                              "  - 1,2,3\n"
                              "  - 1:2, 2:8\n"
                              "  - 1,2,3:8\n"
                              "  - default halaman pertama\n"
                              "    dan seterusnya"), type=str, default="1:")
    parser.add_argument("-i", "--information", dest="info",
                        action="store_true", help="cetak informasi dari item yang dipilih")
    parser.add_argument("--exec", metavar="cmd", dest="_exec",
                        help=("jalankan 'perintah' dengan argument berupa\n"
                              "url yang dipilih.\n\n"
                              "contoh: axel2 {}"))

    valid_e_rules = re.compile(
        r"\[\^.+?\][+*]")
    valid_allDirectRules = valid_e_rules.sub(
        "[A-Za-z0-9-.]{5,7}", Bypass.allBypassPattern.pattern)
    parser.add_argument("--bypass", metavar="url",
                        help=("bypass situs download\n\n"
                              f"contoh url: {regex_to_string(valid_allDirectRules, 3)!r}\n"
                              ))

    parser.add_argument("--list-bypass", action="store_true",
                        help="cetak semua daftar situs download yang dapat\ndi-bypass")

    parser.add_argument("--dump-json", metavar="filename", dest="json_dump",
                        help="simpan hasil ekstraksi unduhan")
    parser.add_argument("--json", action="store_true",
                        help="cetak hasil ekstraksi unduhan")

    network = parser.add_argument_group("Network Options")
    network.add_argument("--proxy", metavar="IP:PORT", type=ipPortType,
                         help="gunakan HTTP/HTTPS proxy")
    network.add_argument("--skip-proxy", action="store_true",
                         help="lewati penggunakan argument --proxy")

    extractor_group = parser.add_argument_group("Daftar Ekstraktor",
                                                description=(
                                                    f"pilih salah satu dari {len(extractors)} situs berikut:"
                                                ))
    extractor_exclusiveGroup = extractor_group.add_mutually_exclusive_group()
    for extractorName, kls in extractors.items():
        extractorName = extractorName.replace("_", "-")
        if hasattr(kls, "host"):
            pa = urlparse(kls.host)
            added = False
            for fullName in [extractorName, re.sub(r"[aioue-]", "", extractorName)]:
                for index in range(0, len(fullName), 2):
                    for shortName in (fullName[index:index+1], fullName[index:index+2]):
                        shortName = shortName.rstrip("-")
                        try:
                            arg = [f"-{shortName}"]
                            if arg[0] == f"-{extractorName}":
                                arg = [f"--{extractorName}"]
                            else:
                                arg.append(f"--{extractorName}")
                            extractor_exclusiveGroup.add_argument(*arg, action="store_true",
                                                                  help=f"site: {pa.scheme}://{pa.netloc} [{kls.tag}]")
                            added = True
                            break
                        except argparse.ArgumentError:
                            continue
                    if added:
                        break
                if added:
                    break
    return parser



from . import BaseExtractor


class Asuka_Zonawibu(BaseExtractor):
    tag = "anime"
    host = "https://asuka.zonawibu.net"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        if (infox := soup.find(class_="contentpost")):

            alias = {
                "english title": "judul_alternatif",
                "genres": "genre",
                "type": "tipe",
                "episodes": "total_episode",
                "producers": "produser",
                "duration": "durasi"
            }
            for p in infox.findAll("p"):
                if ":" not in p.text:
                    continue
                k, v = map(lambda x: x.lower().strip(),
                           self.re.split("\s*:\s*", p.text))
                k = alias.get(k, k)
                if k == "sinopsis":
                    v = p.findNext("p").text
                if v:
                    meta[k] = v

        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        if (direct := soup.find("a", text="DIRECT DOWNLOAD")):
            result[a.text] = a["href"]

        for dl in soup.findAll(class_="dl"):
            for smokeurl in dl.findAll(class_="smokeurl"):
                smokettl = smokeurl.findPrevious(class_="smokettl").text
                if not result.get(smokettl):
                    result[smokettl] = {}

                r = {}
                for a in smokeurl.findAll("a"):
                    r[a.text] = a["href"]
                result[smokettl][smokeurl.strong.text] = r
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}",
                               params={"s": query})
        soup = self.soup(raw)

        result = []
        for postlist in soup.findAll(class_="postlist"):
            a = postlist.a
            r = {
                "id": self.getPath(a["href"]),
                "title": a.text
            }

            for y in postlist.findAll(class_=self.re.compile(r"auth(?:mobile)?")):
                if len(x := y.text.split(":")):
                    r[x[0]] = x[1]

            result.append(r)
        return result



from . import BaseExtractor


class Kuramanime(BaseExtractor):
    tag = None
    host = "https://kuramanime.com"

    def extract_meta(self, id: str) -> dict:
        """
        Ambil semua metadata dari halaman web

        Args:
              id: type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        meta = self.MetaSet()
        self._write(soup)
        return meta

    def extract_data(self, id: str) -> dict:
        """
        Ambil semua situs unduhan dari halaman web

        Args:
              id: jalur url dimulai setelah host, type 'str'
        """

        raw = self.session.get(f"{self.host}/{id}")
        soup = self.soup(raw)

        result = {}
        self._write(soup)
        return result

    def search(self, query: str, page: int = 1) -> list:
        """
        Cari item berdasarkan 'query' yang diberikan

        Args:
              query: kata kunci pencarian, type 'str'
              page: indeks halaman web, type 'int'
        """

        raw = self.session.get(f"{self.host}/page/{page}",
                               params={"s": query})
        soup = self.soup(raw)

        result = []
        for artikel in soup.findAll("article"):
            a = artikel.find("a")
            if not a.img:
                continue
            result.append({
                "title": a["title"],
                "id": self.getPath(a["href"])
            })
        return result
